'use strict';

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _EventConstants = require('./EventConstants');

var _EventConstants2 = _interopRequireDefault(_EventConstants);

var _reactLibEventPluginHub = require('react/lib/EventPluginHub');

var _reactLibEventPluginHub2 = _interopRequireDefault(_reactLibEventPluginHub);

var _reactLibAccumulateInto = require('react/lib/accumulateInto');

var _reactLibAccumulateInto2 = _interopRequireDefault(_reactLibAccumulateInto);

var _reactLibForEachAccumulated = require('react/lib/forEachAccumulated');

var _reactLibForEachAccumulated2 = _interopRequireDefault(_reactLibForEachAccumulated);

var PropagationPhases = _EventConstants2['default'].PropagationPhases;
var getListener = _reactLibEventPluginHub2['default'].getListener;

/**
 * Some event types have a notion of different registration names for different
 * "phases" of propagation. This finds listeners by a given phase.
 */
function listenerAtPhase(id, event, propagationPhase) {
  var registrationName = event.dispatchConfig.phasedRegistrationNames[propagationPhase];
  return getListener(id, registrationName);
}

/**
 * Accumulates without regard to direction, does not look for phased
 * registration names. Same as `accumulateDirectDispatchesSingle` but without
 * requiring that the `dispatchMarker` be the same as the dispatched ID.
 */
function accumulateDispatches(id, ignoredDirection, event) {
  if (event && event.dispatchConfig.registrationName) {
    var registrationName = event.dispatchConfig.registrationName;
    var listener = getListener(id, registrationName);
    if (listener) {
      event._dispatchListeners = (0, _reactLibAccumulateInto2['default'])(event._dispatchListeners, listener);
      event._dispatchIDs = (0, _reactLibAccumulateInto2['default'])(event._dispatchIDs, id);
    }
  }
}

/**
 * Accumulates dispatches on an `SyntheticEvent`, but only for the
 * `dispatchMarker`.
 * @param {SyntheticEvent} event
 */
function accumulateDirectDispatchesSingle(event) {
  if (event && event.dispatchConfig.registrationName) {
    accumulateDispatches(event.dispatchMarker, null, event);
  }
}

function accumulateDirectDispatches(events) {
  (0, _reactLibForEachAccumulated2['default'])(events, accumulateDirectDispatchesSingle);
}

/**
 * A small set of propagation patterns, each of which will accept a small amount
 * of information, and generate a set of "dispatch ready event objects" - which
 * are sets of events that have already been annotated with a set of dispatched
 * listener functions/ids. The API is designed this way to discourage these
 * propagation strategies from actually executing the dispatches, since we
 * always want to collect the entire set of dispatches before executing event a
 * single one.
 *
 * @constructor EventPropagators
 */
var EventPropagators = {
  accumulateDirectDispatches: accumulateDirectDispatches };

module.exports = EventPropagators;