/** @flow */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _EventPropagators = require('./EventPropagators');

var _EventPropagators2 = _interopRequireDefault(_EventPropagators);

var _HardwareManager = require('./HardwareManager');

var _HardwareManager2 = _interopRequireDefault(_HardwareManager);

var _reactLibSyntheticEvent = require('react/lib/SyntheticEvent');

var _reactLibSyntheticEvent2 = _interopRequireDefault(_reactLibSyntheticEvent);

var _reactLibWarning = require('react/lib/warning');

var _reactLibWarning2 = _interopRequireDefault(_reactLibWarning);

var customBubblingEventTypes = _HardwareManager2['default'].customBubblingEventTypes;
var customDirectEventTypes = _HardwareManager2['default'].customDirectEventTypes;

var allTypesByEventName = {};

// TODO: likely remove all bubbling event type references.
/*
for (var bubblingTypeName in customBubblingEventTypes) {
  allTypesByEventName[bubblingTypeName] = customBubblingEventTypes[bubblingTypeName];
}
*/

for (var directTypeName in customDirectEventTypes) {
  /*
  warning(
    !customBubblingEventTypes[directTypeName],
    "Event cannot be both direct and bubbling: %s",
    directTypeName
  );
  */
  allTypesByEventName[directTypeName] = customDirectEventTypes[directTypeName];
}

var HardwareBridgeEventPlugin = {

  // eventTypes: assign({}, customBubblingEventTypes, customDirectEventTypes),
  eventTypes: _extends({}, customDirectEventTypes),

  /**
   * @param {string} topLevelType Record from `EventConstants`.
   * @param {DOMEventTarget} topLevelTarget The listening component root node.
   * @param {string} topLevelTargetID ID of `topLevelTarget`.
   * @param {object} nativeEvent Native browser event.
   * @return {*} An accumulation of synthetic events.
   * @see {EventPluginHub.extractEvents}
   */
  extractEvents: function extractEvents(topLevelType, topLevelTarget, topLevelTargetID, nativeEvent) {
    var bubbleDispatchConfig = null; // customBubblingEventTypes[topLevelType];
    var directDispatchConfig = customDirectEventTypes[topLevelType];
    var event = _reactLibSyntheticEvent2['default'].getPooled(bubbleDispatchConfig || directDispatchConfig, topLevelTargetID, nativeEvent);
    if (bubbleDispatchConfig) {
      _EventPropagators2['default'].accumulateTwoPhaseDispatches(event);
    } else if (directDispatchConfig) {
      _EventPropagators2['default'].accumulateDirectDispatches(event);
    } else {
      return null;
    }
    return event;
  }
};

exports['default'] = HardwareBridgeEventPlugin;
module.exports = exports['default'];