/*eslint no-console:0*/
'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _WRITE_TYPE;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _defineProperty(obj, key, value) { return Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); }

var _firmata = require('firmata');

// import ReactHardwareTagHandles from './ReactHardwareTagHandles';

var _reactLibObjectAssign = require('react/lib/Object.assign');

var _reactLibObjectAssign2 = _interopRequireDefault(_reactLibObjectAssign);

var _componentsInputModes = require('../components/inputModes');

var _componentsInputModes2 = _interopRequireDefault(_componentsInputModes);

var _reactLibWarning = require('react/lib/warning');

var _reactLibWarning2 = _interopRequireDefault(_reactLibWarning);

var _reactLibInvariant = require('react/lib/invariant');

var _reactLibInvariant2 = _interopRequireDefault(_reactLibInvariant);

var _pinMappingsArduinoUno = require('./pinMappings/ArduinoUno');

var _pinMappingsArduinoUno2 = _interopRequireDefault(_pinMappingsArduinoUno);

var _customEventTypes = require('./customEventTypes');

var capitalize = function capitalize(w) {
  return '' + w[0].toUpperCase() + '' + w.slice(1);
};

var WRITE_TYPE = (_WRITE_TYPE = {}, _defineProperty(_WRITE_TYPE, 0, 'digital'), _defineProperty(_WRITE_TYPE, 1, 'digital'), _defineProperty(_WRITE_TYPE, 2, 'analog'), _defineProperty(_WRITE_TYPE, 3, 'analog'), _defineProperty(_WRITE_TYPE, 4, 'servo'), _WRITE_TYPE);

// TODO: need interfaces for firmata, node-hid, etc.

var Registry = {
  children: [] };

var noop = function noop() {};
var HardwareManager = {
  createConnection: function createConnection(containerTag) {
    var callback = arguments[1] === undefined ? noop : arguments[1];

    var onConnect = function onConnect(err) {
      if (err) {
        throw err;
      }
      console.log('Connected to %s', containerTag);
      // console.log(Registry[containerTag].board)
      callback();
    };

    if (Registry[containerTag]) {
      if (Registry[containerTag].board.isReady) {
        callback();
      }
    } else {
      Registry[containerTag] = {
        board: new _firmata.Board(containerTag, onConnect),
        children: [] };

      Registry.board = Registry[containerTag].board;
    }

    return Registry[containerTag];
  },

  manageChildren: function manageChildren(componentTag,
  // TODO (remove): most of these are unnecessary in the hardware environment
  moveFromIndices, moveToIndices, addChildTags, addAtIndices, removeAtIndices) {
    console.log('TODO: HardwareManager#manageChildren');
  },

  createView: function createView(tag, name, payload) {
    if (!payload || typeof payload.pin === 'undefined') {
      (0, _reactLibWarning2['default'])(name === 'Board', 'A component `%s` must have a pin to render.', name);
      return;
    }

    Registry.children[tag] = {
      name: name,
      props: payload };

    // analog pins
    if (typeof payload.pin === 'string') {
      payload.physicalPin = _pinMappingsArduinoUno2['default'][payload.pin];
      Registry.board.pinMode(_pinMappingsArduinoUno2['default'][payload.pin], payload.mode);
    } else {
      Registry.board.pinMode(payload.pin, payload.mode);
    }

    if (typeof payload.value !== 'undefined') {
      Registry.board['' + WRITE_TYPE[payload.mode] + 'Write'](payload.pin, payload.value);
    }
    console.log('create', payload);
  },

  updateView: function updateView(tag, _name, payload) {
    var _Registry$children$tag = Registry.children[tag];
    var name = _Registry$children$tag.name;
    var props = _Registry$children$tag.props;

    (0, _reactLibInvariant2['default'])(name === _name, 'It appears like you’re trying to update a view in pin %s to a new ' + 'component type.', tag, name, _name);

    // TODO: Make this much less ugly
    if (typeof payload.mode !== 'undefined') {
      Registry.board.pinMode(props.pin, payload.mode);
      props.mode = payload.mode;
    }

    if (typeof payload.value !== 'undefined') {
      // console.log(`${WRITE_TYPE[props.mode]}Write`, props.pin, payload.value);
      Registry.board['' + WRITE_TYPE[props.mode] + 'Write'](props.pin, payload.value);
    }

    (0, _reactLibObjectAssign2['default'])(props, payload);
  },

  read: function read(tag, callback) {
    var props = Registry.children[tag].props;

    Registry.children[tag].readListener = callback;
    var method = '' + WRITE_TYPE[props.mode] + 'Read';

    if (props.mode === _componentsInputModes2['default'].ANALOG) {
      // TODO: this mapping needs to be smarter, but oh well
      Registry.board[method](props.physicalPin - _pinMappingsArduinoUno2['default'][props.pin], callback);
    } else {
      Registry.board[method](props.pin, callback);
    }
  },

  destroyRead: function destroyRead(tag) {
    var _Registry$children$tag2 = Registry.children[tag];
    var props = _Registry$children$tag2.props;
    var readListener = _Registry$children$tag2.readListener;

    Registry.board['report' + capitalize(WRITE_TYPE[props.mode]) + 'Pin'](props.pin, 0);
    Registry.board.removeListener('digital-read-' + props.pin, readListener);
  },

  measure: function measure(tag, callback) {
    console.log('TODO: HardwareManager.measure');
  },

  setJSResponder: function setJSResponder(tag) {
    console.log('TODO: HardwareManager#setJSResponder to %s', tag);
  },

  clearJSResponder: function clearJSResponder() {
    console.log('TODO: HardwareManager#clearJSResponder');
  },

  customDirectEventTypes: _customEventTypes.customDirectEventTypes };

exports['default'] = HardwareManager;
module.exports = exports['default'];
// input
//output
// analog
// pwm
// servo