/* @flow */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _reactLibEventPluginHub = require('react/lib/EventPluginHub');

var _reactLibEventPluginHub2 = _interopRequireDefault(_reactLibEventPluginHub);

var _reactLibReactEventEmitterMixin = require('react/lib/ReactEventEmitterMixin');

var _reactLibReactEventEmitterMixin2 = _interopRequireDefault(_reactLibReactEventEmitterMixin);

var _ReactHardwareTagHandles = require('./ReactHardwareTagHandles');

var _ReactHardwareTagHandles2 = _interopRequireDefault(_ReactHardwareTagHandles);

var _NodeHandle = require('./NodeHandle');

var _NodeHandle2 = _interopRequireDefault(_NodeHandle);

var _EventConstants = require('./EventConstants');

var _EventConstants2 = _interopRequireDefault(_EventConstants);

var _reactLibObjectAssign = require('react/lib/Object.assign');

var _reactLibObjectAssign2 = _interopRequireDefault(_reactLibObjectAssign);

var _reactLibWarning = require('react/lib/warning');

var _reactLibWarning2 = _interopRequireDefault(_reactLibWarning);

var topLevelTypes = _EventConstants2['default'].topLevelTypes;

/**
 * Version of `ReactBrowserEventEmitter` that works on the receiving side of a
 * serialized worker boundary. Lulz there is no worker boundary.
 */

// Shared default empty native event - conserve memory.
var EMPTY_NATIVE_EVENT = {};

/**
 * `ReactHardwareEventEmitter` is used to attach top-level event listeners. For
 * example:
 *
 *   ReactHardwareEventEmitter.putListener('myID', 'onDown', myFunction);
 *
 * This would allocate a "registration" of `('onDown', myFunction)` on 'myID'.
 *
 * @internal
 */
var ReactHardwareEventEmitter = (0, _reactLibObjectAssign2['default'])({}, _reactLibReactEventEmitterMixin2['default'], {

  registrationNames: _reactLibEventPluginHub2['default'].registrationNameModules,

  putListener: _reactLibEventPluginHub2['default'].putListener,

  getListener: _reactLibEventPluginHub2['default'].getListener,

  deleteListener: _reactLibEventPluginHub2['default'].deleteListener,

  deleteAllListeners: _reactLibEventPluginHub2['default'].deleteAllListeners,

  /**
   * Internal version of `receiveEvent` in terms of normalized (non-tag)
   * `rootNodeID`.
   *
   * @see receiveEvent.
   *
   * @param {rootNodeID} rootNodeID React root node ID that event occured on.
   * @param {TopLevelType} topLevelType Top level type of event.
   * @param {object} nativeEventParam Object passed from native.
   */
  _receiveRootNodeIDEvent: function _receiveRootNodeIDEvent(rootNodeID, topLevelType, nativeEventParam) {
    var nativeEvent = nativeEventParam || EMPTY_NATIVE_EVENT;
    ReactHardwareEventEmitter.handleTopLevel(topLevelType, rootNodeID, rootNodeID, nativeEvent);
  },

  /**
   * Publically exposed method on module for native objc to invoke when a top
   * level event is extracted.
   * @param {rootNodeID} rootNodeID React root node ID that event occured on.
   * @param {TopLevelType} topLevelType Top level type of event.
   * @param {object} nativeEventParam Object passed from native.
   */
  receiveEvent: function receiveEvent(tag, topLevelType, nativeEventParam) {
    var rootNodeID = _ReactHardwareTagHandles2['default'].tagToRootNodeID[tag];
    ReactHardwareEventEmitter._receiveRootNodeIDEvent(rootNodeID, topLevelType, nativeEventParam);
  },

  /**
   */
  receiveReads: function receiveReads(eventTopLevelType, reads, changedIndices) {
    console.log('TODO: receiveReads: %s', eventTopLevelType, reads);
    /*
    var changedReads =
      eventTopLevelType === topLevelTypes.topTouchEnd ||
      eventTopLevelType === topLevelTypes.topTouchCancel ?
      removeTouchesAtIndices(touches, changedIndices) :
      touchSubsequence(touches, changedIndices);
     for (var jj = 0; jj < changedTouches.length; jj++) {
      var touch = changedTouches[jj];
      // Touch objects can fullfill the role of `DOM` `Event` objects if we set
      // the `changedTouches`/`touches`. This saves allocations.
      touch.changedTouches = changedTouches;
      touch.touches = touches;
      var nativeEvent = touch;
      var rootNodeID = null;
      var target = nativeEvent.target;
      if (target !== null && target !== undefined) {
        if (target < ReactHardwareTagHandles.tagsStartAt) {
          if (__DEV__) {
            warning(
              false,
              'A view is reporting that a touch occured on tag zero.'
            );
          }
        } else {
          rootNodeID = NodeHandle.getRootNodeID(target);
        }
      }
      ReactHardwareEventEmitter._receiveRootNodeIDEvent(
        rootNodeID,
        eventTopLevelType,
        nativeEvent
      );
    }
    */
  }
});

exports['default'] = ReactHardwareEventEmitter;
module.exports = exports['default'];