/*eslint no-console:0, no-use-before-define:0*/

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _ReactHardwareTagHandles = require('./ReactHardwareTagHandles');

var _ReactHardwareTagHandles2 = _interopRequireDefault(_ReactHardwareTagHandles);

var _reactLibReactReconciler = require('react/lib/ReactReconciler');

var _reactLibReactReconciler2 = _interopRequireDefault(_reactLibReactReconciler);

var _reactLibReactUpdateQueue = require('react/lib/ReactUpdateQueue');

var _reactLibReactUpdateQueue2 = _interopRequireDefault(_reactLibReactUpdateQueue);

var _reactLibReactUpdates = require('react/lib/ReactUpdates');

var _reactLibReactUpdates2 = _interopRequireDefault(_reactLibReactUpdates);

var _reactLibInstantiateReactComponent = require('react/lib/instantiateReactComponent');

var _reactLibInstantiateReactComponent2 = _interopRequireDefault(_reactLibInstantiateReactComponent);

var _reactLibEmptyObject = require('react/lib/emptyObject');

var _reactLibEmptyObject2 = _interopRequireDefault(_reactLibEmptyObject);

var _reactLibShouldUpdateReactComponent = require('react/lib/shouldUpdateReactComponent');

var _reactLibShouldUpdateReactComponent2 = _interopRequireDefault(_reactLibShouldUpdateReactComponent);

var _HardwareManager = require('./HardwareManager');

var _HardwareManager2 = _interopRequireDefault(_HardwareManager);

var _reactLibInvariant = require('react/lib/invariant');

var _reactLibInvariant2 = _interopRequireDefault(_reactLibInvariant);

/*
var ReactPerf = require('ReactPerf');
*/

var instanceNumberToChildRootID = function instanceNumberToChildRootID(rootNodeID, instanceNumber) {
  return '' + rootNodeID + '[' + instanceNumber + ']';
};

/**
 * Mounts this component and inserts it into the DOM.
 *
 * @param {ReactComponent} componentInstance The instance to mount.
 * @param {number} rootID ID of the root node.
 * @param {number} container container element to mount into.
 * @param {ReactReconcileTransaction} transaction
 */
function mountComponentIntoNode(componentInstance, rootID, container, transaction) {
  var markup = _reactLibReactReconciler2['default'].mountComponent(componentInstance, rootID, transaction, _reactLibEmptyObject2['default']);
  componentInstance._isTopLevel = true;
  ReactHardwareMount._mountImageIntoNode(markup, container);
}

/**
 * Batched mount.
 *
 * @param {ReactComponent} componentInstance The instance to mount.
 * @param {number} rootID ID of the root node.
 * @param {number} container container element to mount into.
 */
function batchedMountComponentIntoNode(componentInstance, rootID, container) {
  var transaction = _reactLibReactUpdates2['default'].ReactReconcileTransaction.getPooled();
  transaction.perform(mountComponentIntoNode, null, componentInstance, rootID, container, transaction);
  _reactLibReactUpdates2['default'].ReactReconcileTransaction.release(transaction);
}

/**
 * As soon as `ReactMount` is refactored to not rely on the DOM, we can share
 * code between the two. For now, we'll hard code the ID logic.
 */
var ReactHardwareMount = {
  instanceCount: 0,

  _instancesByContainerID: {},

  renderComponent: function renderComponent(nextElement, containerTag, callback) {
    _HardwareManager2['default'].createConnection(containerTag, function () {
      var topRootNodeID = _ReactHardwareTagHandles2['default'].tagToRootNodeID[containerTag];
      if (topRootNodeID) {
        var prevComponent = ReactHardwareMount._instancesByContainerID[topRootNodeID];
        if (prevComponent) {
          var prevElement = prevComponent._currentElement;
          if ((0, _reactLibShouldUpdateReactComponent2['default'])(prevElement, nextElement)) {
            _reactLibReactUpdateQueue2['default'].enqueueElementInternal(prevComponent, nextElement);
            if (callback) {
              _reactLibReactUpdateQueue2['default'].enqueueCallbackInternal(prevComponent, callback);
            }
            return prevComponent;
          } else {
            ReactHardwareMount.unmountComponentAtNode(containerTag);
          }
        }
      }

      (0, _reactLibInvariant2['default'])(_ReactHardwareTagHandles2['default'].reactTagIsHardwareTopRootID(containerTag), 'You cannot render into anything but a SerialPort connection.');

      topRootNodeID = _ReactHardwareTagHandles2['default'].allocateRootNodeIDForTag(containerTag);
      _ReactHardwareTagHandles2['default'].associateRootNodeIDWithMountedNodeHandle(topRootNodeID, containerTag);

      var instance = (0, _reactLibInstantiateReactComponent2['default'])(nextElement);
      ReactHardwareMount._instancesByContainerID[topRootNodeID] = instance;

      _HardwareManager2['default'].createConnection(containerTag);

      var childRootNodeID = instanceNumberToChildRootID(topRootNodeID, ReactHardwareMount.instanceCount++);

      // TODO: make the initial render asynchrounous as well
      // The initial render is synchronous but any updates that happen during
      // rendering, in componentWillMount or componentDidMount, will be batched
      // according to the current batching strategy.

      _reactLibReactUpdates2['default'].batchedUpdates(batchedMountComponentIntoNode, instance, childRootNodeID, topRootNodeID);
      var component = instance.getPublicInstance();
      if (callback) {
        callback.call(component);
      }
      return component;
    });
  },

  /**
   * Unmount component at container ID by iterating through each child component
   * that has been rendered and unmounting it. There should just be one child
   * component at this time.
   */
  unmountComponentAtNode: function unmountComponentAtNode(containerTag) {
    if (!_ReactHardwareTagHandles2['default'].reactTagIsHardwareTopRootID(containerTag)) {
      console.error('You cannot render into anything but a top root');
      return false;
    }

    var containerID = _ReactHardwareTagHandles2['default'].tagToRootNodeID[containerTag];
    var instance = ReactHardwareMount._instancesByContainerID[containerID];
    if (!instance) {
      return false;
    }
    ReactHardwareMount.unmountComponentFromNode(instance, containerID);
    delete ReactHardwareMount._instancesByContainerID[containerID];
    return true;
  },

  getNode: function getNode(id) {
    return id;
  },

  /**
   * @param {view} mountImage view tree image
   * @param {string} containerID node to insert sub-view into
   */
  _mountImageIntoNode: function _mountImageIntoNode(mountImage, containerID) {
    _ReactHardwareTagHandles2['default'].associateRootNodeIDWithMountedNodeHandle(mountImage.rootNodeID, mountImage.tag);
    var addChildTags = [mountImage.tag];
    var addAtIndices = [0];

    // TODO: simplify this to just adding. Nothing can be removed.
    _HardwareManager2['default'].manageChildren(_ReactHardwareTagHandles2['default'].mostRecentMountedNodeHandleForRootNodeID(containerID), null, // moveFromIndices
    null, // moveToIndices
    addChildTags, addAtIndices, null // removeAtIndices
    );
  } };

exports['default'] = ReactHardwareMount;
module.exports = exports['default'];