/*eslint no-console:0*/
'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _HardwareMethodsMixin = require('./HardwareMethodsMixin');

var _HardwareMethodsMixin2 = _interopRequireDefault(_HardwareMethodsMixin);

var _ReactHardwareComponentMixin = require('./ReactHardwareComponentMixin');

var _ReactHardwareComponentMixin2 = _interopRequireDefault(_ReactHardwareComponentMixin);

var _ReactHardwareEventEmitter = require('./ReactHardwareEventEmitter');

var _ReactHardwareEventEmitter2 = _interopRequireDefault(_ReactHardwareEventEmitter);

var _ReactHardwareTagHandles = require('./ReactHardwareTagHandles');

var _ReactHardwareTagHandles2 = _interopRequireDefault(_ReactHardwareTagHandles);

var _reactLibReactMultiChild = require('react/lib/ReactMultiChild');

var _reactLibReactMultiChild2 = _interopRequireDefault(_reactLibReactMultiChild);

var _vendorDiffRawProperties = require('./vendor/diffRawProperties');

var _vendorDiffRawProperties2 = _interopRequireDefault(_vendorDiffRawProperties);

var _reactLibWarning = require('react/lib/warning');

var _reactLibWarning2 = _interopRequireDefault(_reactLibWarning);

var _vendorDeepFreezeAndThrowOnMutationInDev = require('./vendor/deepFreezeAndThrowOnMutationInDev');

var _vendorDeepFreezeAndThrowOnMutationInDev2 = _interopRequireDefault(_vendorDeepFreezeAndThrowOnMutationInDev);

var _HardwareManager = require('./HardwareManager');

var _HardwareManager2 = _interopRequireDefault(_HardwareManager);

var _reactLibObjectAssign = require('react/lib/Object.assign');

var _reactLibObjectAssign2 = _interopRequireDefault(_reactLibObjectAssign);

var __DEV__ = true;
var registrationNames = _ReactHardwareEventEmitter2['default'].registrationNames;
var putListener = _ReactHardwareEventEmitter2['default'].putListener;
var deleteAllListeners = _ReactHardwareEventEmitter2['default'].deleteAllListeners;

/**
 * @constructor ReactHardwareNativeComponent
 * @extends ReactComponent
 * @extends ReactMultiChild
 * @param {!object} viewConfig View Configuration.
 */
var ReactHardwareNativeComponent = function ReactHardwareNativeComponent(viewConfig) {
  this.viewConfig = viewConfig;
};

/**
 * Generates and caches arrays of the form:
 *
 *    [0, 1, 2, 3]
 *    [0, 1, 2, 3, 4]
 *    [0, 1]
 *
 * @param {number} size Size of array to generate.
 * @return {Array<number>} Array with values that mirror the index.
 */
var cachedIndexArray = function cachedIndexArray(size) {
  var cachedResult = cachedIndexArray._cache[size];
  if (!cachedResult) {
    var arr = [];
    for (var i = 0; i < size; i++) {
      arr[i] = i;
    }
    cachedIndexArray._cache[size] = arr;
    return cachedIndexArray._cache[size];
  } else {
    return cachedResult;
  }
};
cachedIndexArray._cache = {};

/**
 * Mixin for containers that contain UIViews. NOTE: markup is rendered markup
 * which is a `viewID` ... see the return value for `mountComponent` !
 */
ReactHardwareNativeComponent.Mixin = {
  getPublicInstance: function getPublicInstance() {
    // TODO: This should probably use a composite wrapper
    return this;
  },

  construct: function construct(element) {
    this._currentElement = element;
  },

  unmountComponent: function unmountComponent() {
    deleteAllListeners(this._rootNodeID);
    this.unmountChildren();
    this._rootNodeID = null;
  },

  /**
   * Every native component is responsible for allocating its own `tag`, and
   * issuing the native `createView` command. But it is not responsible for
   * recording the fact that its own `rootNodeID` is associated with a
   * `nodeHandle`. Only the code that actually adds its `nodeHandle` (`tag`) as
   * a child of a container can confidently record that in
   * `ReactHardwareTagHandles`.
   */
  initializeChildren: function initializeChildren(children, containerTag, transaction, context) {
    var mountImages = this.mountChildren(children, transaction, context);
    // In a well balanced tree, half of the nodes are in the bottom row and have
    // no children - let's avoid calling out to the native bridge for a large
    // portion of the children.
    if (mountImages.length) {
      var indexes = cachedIndexArray(mountImages.length);
      // TODO: Pool these per platform view class. Reusing the `mountImages`
      // array would likely be a jit deopt.
      var createdTags = [];
      for (var i = 0; i < mountImages.length; i++) {
        var mountImage = mountImages[i];
        var childTag = mountImage.tag;
        var childID = mountImage.rootNodeID;
        (0, _reactLibWarning2['default'])(mountImage && mountImage.rootNodeID && mountImage.tag, 'Mount image returned does not have required data');
        _ReactHardwareTagHandles2['default'].associateRootNodeIDWithMountedNodeHandle(childID, childTag);
        createdTags[i] = mountImage.tag;
      }
      _HardwareManager2['default'].manageChildren(containerTag, null, null, createdTags, indexes, null);
    }
  },

  /**
   *
   * @param {!object} prevProps Previous properties
   * @param {!object} nextProps Next properties
   * @param {!object} validAttributes Set of valid attributes and how they
   *                  should be diffed
   */
  computeUpdatedProperties: function computeUpdatedProperties(prevProps, nextProps, validAttributes) {
    if (__DEV__) {
      for (var key in nextProps) {
        if (nextProps.hasOwnProperty(key) && nextProps[key] && validAttributes[key]) {
          (0, _vendorDeepFreezeAndThrowOnMutationInDev2['default'])(nextProps[key]);
        }
      }
    }

    var updatePayload = (0, _vendorDiffRawProperties2['default'])(null, // updatePayload
    prevProps, nextProps, validAttributes);

    return updatePayload;
  },

  /**
   * Updates the component's currently mounted representation.
   *
   * @param {object} nextElement
   * @param {ReactReconcileTransaction} transaction
   * @param {object} context
   * @internal
   */
  receiveComponent: function receiveComponent(nextElement, transaction, context) {
    var prevElement = this._currentElement;
    this._currentElement = nextElement;

    var updatePayload = this.computeUpdatedProperties(prevElement.props, nextElement.props, this.viewConfig.validAttributes);

    if (updatePayload) {
      _HardwareManager2['default'].updateView(_ReactHardwareTagHandles2['default'].mostRecentMountedNodeHandleForRootNodeID(this._rootNodeID), this.viewConfig.uiViewClassName, updatePayload);
    }

    this._reconcileListenersUponUpdate(prevElement.props, nextElement.props);
    this.updateChildren(nextElement.props.children, transaction, context);
  },

  /**
   * @param {object} initialProps Native component props.
   */
  _registerListenersUponCreation: function _registerListenersUponCreation(initialProps) {
    for (var key in initialProps) {
      // NOTE: The check for `!props[key]`, is only possible because this method
      // registers listeners the *first* time a component is created.
      if (registrationNames[key] && initialProps[key]) {
        var listener = initialProps[key];
        putListener(this._rootNodeID, key, listener);
      }
    }
  },

  /**
   * Reconciles event listeners, adding or removing if necessary.
   * @param {object} prevProps Native component props including events.
   * @param {object} nextProps Next native component props including events.
   */
  _reconcileListenersUponUpdate: function _reconcileListenersUponUpdate(prevProps, nextProps) {
    for (var key in nextProps) {
      if (registrationNames[key] && nextProps[key] != prevProps[key]) {
        putListener(this._rootNodeID, key, nextProps[key]);
      }
    }
  },

  /**
   * @param {string} rootID Root ID of this subtree.
   * @param {Transaction} transaction For creating/updating.
   * @return {string} Unique hardware view tag.
   */
  mountComponent: function mountComponent(rootID, transaction, context) {
    this._rootNodeID = rootID;

    var tag = _ReactHardwareTagHandles2['default'].allocateTag();

    var updatePayload = this.computeUpdatedProperties({}, // previous props
    this._currentElement.props, // next props
    this.viewConfig.validAttributes);
    _HardwareManager2['default'].createView(tag, this.viewConfig.uiViewClassName, updatePayload);

    this._registerListenersUponCreation(this._currentElement.props);
    this.initializeChildren(this._currentElement.props.children, tag, transaction, context);
    return {
      rootNodeID: rootID,
      tag: tag };
  } };

/**
 * Order of mixins is important. ReactHardwareNativeComponent overrides methods in
 * ReactMultiChild.
 */
(0, _reactLibObjectAssign2['default'])(ReactHardwareNativeComponent.prototype, _reactLibReactMultiChild2['default'].Mixin, ReactHardwareNativeComponent.Mixin, _HardwareMethodsMixin2['default'], _ReactHardwareComponentMixin2['default']);

exports['default'] = ReactHardwareNativeComponent;
module.exports = exports['default'];