'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _reactLibCallbackQueue = require('react/lib/CallbackQueue');

var _reactLibCallbackQueue2 = _interopRequireDefault(_reactLibCallbackQueue);

var _reactLibPooledClass = require('react/lib/PooledClass');

var _reactLibPooledClass2 = _interopRequireDefault(_reactLibPooledClass);

var _reactLibTransaction = require('react/lib/Transaction');

var _reactLibTransaction2 = _interopRequireDefault(_reactLibTransaction);

var _reactLibObjectAssign = require('react/lib/Object.assign');

var _reactLibObjectAssign2 = _interopRequireDefault(_reactLibObjectAssign);

/**
 * Provides a `CallbackQueue` queue for collecting `onDOMReady` callbacks during
 * the performing of the transaction.
 */
var ON_DOM_READY_QUEUEING = {
  /**
   * Initializes the internal `onDOMReady` queue.
   */
  initialize: function initialize() {
    this.reactMountReady.reset();
  },

  /**
   * After DOM is flushed, invoke all registered `onDOMReady` callbacks.
   */
  close: function close() {
    this.reactMountReady.notifyAll();
  } };

/**
 * Executed within the scope of the `Transaction` instance. Consider these as
 * being member methods, but with an implied ordering while being isolated from
 * each other.
 */
var TRANSACTION_WRAPPERS = [ON_DOM_READY_QUEUEING];

function ReactHardwareReconcileTransaction() {
  this.reinitializeTransaction();
  this.reactMountReady = _reactLibCallbackQueue2['default'].getPooled(null);
}

var Mixin = {
  /**
   * @see Transaction
   * @abstract
   * @final
   * @return {array<object>} List of operation wrap proceedures.
   *   TODO: convert to array<TransactionWrapper>
   */
  getTransactionWrappers: function getTransactionWrappers() {
    return TRANSACTION_WRAPPERS;
  },

  /**
   * @return {object} The queue to collect `onDOMReady` callbacks with.
   *   TODO: convert to ReactMountReady
   */
  getReactMountReady: function getReactMountReady() {
    return this.reactMountReady;
  },

  /**
   * `PooledClass` looks for this, and will invoke this before allowing this
   * instance to be resused.
   */
  destructor: function destructor() {
    _reactLibCallbackQueue2['default'].release(this.reactMountReady);
    this.reactMountReady = null;
  } };

(0, _reactLibObjectAssign2['default'])(ReactHardwareReconcileTransaction.prototype, _reactLibTransaction2['default'].Mixin, ReactHardwareReconcileTransaction, Mixin);

_reactLibPooledClass2['default'].addPoolingTo(ReactHardwareReconcileTransaction);

exports['default'] = ReactHardwareReconcileTransaction;
module.exports = exports['default'];